package com.nisum.spark.dem.Consumer;

import com.nisum.spark.dem.ProductInfo.ProductsDto;
import lombok.extern.slf4j.Slf4j;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.types.StructType;

import java.util.concurrent.TimeoutException;

@Slf4j
public class StreamingConsumer {
    public static final String MONGODB_DOCUMENT_COLLECTION = "mongodb://127.0.0.1/ProductsDB.product_list_status_shipped";
    private static String kafkaBootstrapServers = "localhost:9092";
    private static String kafkaTopic = "TP.WEBFLUX.KAFKA1";

    public static void main(String[] args) {
        log.info("Entered into main method to start StreamingConsumer: ");

        SparkSession spark = getSparkSession();

        Dataset<Row> readData = consumeDatafromKafka(spark);

        Dataset<Row> filteredDataFrame = filteredData();

        persistConsumedDataToMongoDB();
    }

    private static SparkSession getSparkSession() {
        log.info("Create and get Spark session: ");

        SparkSession spark = SparkSession.builder()
                .appName("DataStreamingWithSparkMongoDB")
                .master("local[*]")
                .config("spark.mongodb.input.uri", MONGODB_DOCUMENT_COLLECTION)
                .config("spark.mongodb.output.uri", MONGODB_DOCUMENT_COLLECTION)
                .getOrCreate();
        return spark;
    }
    private static Dataset<Row> consumeDatafromKafka(SparkSession spark) {
        log.info("Read data streams as a streaming DataFrame: ");

        Dataset<Row> dataFrame = spark.readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers", kafkaBootstrapServers)
                .option("subscribe", kafkaTopic)
                .option("startingOffsets", "earliest")
                .option("maxOffsetsPerTrigger", 100)
                .load();
        log.debug("Read data streams: "+dataFrame.toString());
        return dataFrame;
    }

    private static Dataset<Row> filteredData() {
        log.info("Filtering dataframe to fetch required data: ");

        StructType schema = Encoders.bean(ProductsDto.class)
                .schema();

        Dataset<Row> readWrite = consumeDatafromKafka(getSparkSession()).selectExpr("CAST(value AS STRING) AS json")
                .select(functions.from_json(functions.col("json"), schema)
                                .as("data"))
                .select("data.*");
        //Dataset<Row> filteredData = readWrite.filter(readWrite.col("deliveryStatus").equalTo("Shipped"));

        String startTime = "2023-05-26T10:24:00.000Z";
        String endTime = "2023-05-26T12:24:00.000Z";

        log.info("Filter the DataFrame based on the status and timeStamp: ");
        Dataset<Row> filteredData = readWrite.filter(readWrite.col("deliveryStatus")
                                                             .equalTo("Shipped"))
                .filter(readWrite.col("timeStamp")
                                .geq(startTime)
                                .and(readWrite.col("timeStamp")
                                             .leq(endTime)));

        return filteredData;
    }

    private static void persistConsumedDataToMongoDB() {

        log.info("Persist the filtered data to MongoDB: ");
        StreamingQuery query = null;
        try {
            query = filteredData().writeStream()
                    .foreachBatch((Dataset<Row> batch, Long batchId) -> {
                        log.info("Write each batch to MongoDB: ");
                        batch.write()
                                .format("com.mongodb.spark.sql.DefaultSource")
                                .mode("append") // Specify the write mode as needed (e.g., append, overwrite)
                                .option("database", "ProductsDB") // MongoDB database name
                                .option("collection", "product_list_status_shipped") // MongoDB collection name
                                .save();
                    })
                    .start();
            query.awaitTermination();
        } catch (TimeoutException | StreamingQueryException exception) {
            exception.printStackTrace();
        }
    }
}
